/**
 * Created by xieli on 2017/2/13.
 */

module game {

    export class ValueType {
        /** !#en This method returns an exact copy of current value.
         !#zh 克隆当前值，该方法返回一个新对象，新对象的值和原对象相等。
         */
        clone(): ValueType {
            return cc.clone(this);
        }
        /** !#en Compares this object with the other one.
         !#zh 当前对象是否等于指定对象。
         */

        equals(other: ValueType): boolean {
            //need zhaofei fex
           if(other === this){
               return true;
           }else{
               return false;
           }
        }

        /** !#en TODO
         !#zh 转换为方便阅读的字符串。
         */
        toString(): string {
            return JSON.stringify(this);
        }

        /** !#en
         Linearly interpolates between this value to to value by ratio which is in the range [0, 1].
         When ratio = 0 returns this. When ratio = 1 return to. When ratio = 0.5 returns the average of this and to.
         !#zh
         线性插值。<br/>
         当 ratio = 0 时返回自身，ratio = 1 时返回目标，ratio = 0.5 返回自身和目标的平均值。。
         @param to the to value
         @param ratio the interpolation coefficient
         */
        // lerp(to: ValueType, ratio: number): ValueType {
        //     //need zhaofei fex
        //     switch(ratio){
        //         case 0:return to;
        //         //case 0.5:return 0;
        //         case 1:return this;
        //     }
        //     return null;
        // }
    }
    /** !#en Representation of 2D vectors and points.
     !#zh 表示 2D 向量和坐标 */
    export class Vec2 extends ValueType {

        x: number;
        y: number;

        /** !#en
         Constructor
         see {{#crossLink "cc/vec2:method"}}cc.v2{{/crossLink}} or {{#crossLink "cc/p:method"}}cc.p{{/crossLink}}
         !#zh
         构造函数，可查看 {{#crossLink "cc/vec2:method"}}cc.v2{{/crossLink}} 或者 {{#crossLink "cc/p:method"}}cc.p{{/crossLink}}
         */



        constructor(x?: number, y?: number) {
            super();
            this.x = x;
            this.y = y;
        }


        // /** !#en clone a Vec2 value
        //  !#zh 克隆一个 Vec2 值
        //  */
        // clone(): Vec2;

        /** !#en TODO
         !#zh 设置向量值。
         @param newValue !#en new value to set. !#zh 要设置的新值
         */
        set(x: number, y: number): Vec2 {
            this.x = x;
            this.y = y;
            return this;
        }

        /** !#en TODO
         !#zh 当前的向量是否与指定的向量相等。
         @param other !#en
         Constructor
         see {{#crossLink "cc/vec2:method"}}cc.v2{{/crossLink}} or {{#crossLink "cc/p:method"}}cc.p{{/crossLink}}
         !#zh
         构造函数，可查看 {{#crossLink "cc/vec2:method"}}cc.v2{{/crossLink}} 或者 {{#crossLink "cc/p:method"}}cc.p{{/crossLink}}
         */
        equals(other: Vec2): boolean
        {
            if(this.x == other.x && this.y == other.y){
                return true;
            }
            return false;
        }

        /** !#en TODO
         !#zh 转换为方便阅读的字符串。
         */
        toString(): string{
            return JSON.stringify(this);
        }

        /** !#en TODO
         !#zh 线性插值。
         @param to !#en
         Constructor
         see {{#crossLink "cc/vec2:method"}}cc.v2{{/crossLink}} or {{#crossLink "cc/p:method"}}cc.p{{/crossLink}}
         !#zh
         构造函数，可查看 {{#crossLink "cc/vec2:method"}}cc.v2{{/crossLink}} 或者 {{#crossLink "cc/p:method"}}cc.p{{/crossLink}}
         @param ratio the interpolation coefficient
         @param out optional, the receiving vector
         */
        lerp(to: Vec2, ratio: number, out: Vec2): Vec2{
            //return cc.pLerp(to,out,ratio);
            var f = cc.pLerp(cc.p(to.x,to.y),cc.p(out.x,out.y),ratio);
            out.x = f.x;
            out.y = f.y;
            return out;
        }

        /** !#en Adds this vector. If you want to save result to another vector, use add() instead.
         !#zh 向量加法。如果你想保存结果到另一个向量，使用 add() 代替。
         @param vector !#en
         Constructor
         see {{#crossLink "cc/vec2:method"}}cc.v2{{/crossLink}} or {{#crossLink "cc/p:method"}}cc.p{{/crossLink}}
         !#zh
         构造函数，可查看 {{#crossLink "cc/vec2:method"}}cc.v2{{/crossLink}} 或者 {{#crossLink "cc/p:method"}}cc.p{{/crossLink}}

         @example
         ```js
         var v = cc.v2(10, 10);
         v.addSelf(cc.v2(5, 5));// return Vec2 {x: 15, y: 15};
         ```
         */
        addSelf(vector: Vec2): Vec2{

           this.x += vector.x;
           this.y += vector.y;
           return this;
        }

        /** !#en Adds two vectors, and returns the new result.
         !#zh 向量加法，并返回新结果。
         @param vector !#en
         Constructor
         see {{#crossLink "cc/vec2:method"}}cc.v2{{/crossLink}} or {{#crossLink "cc/p:method"}}cc.p{{/crossLink}}
         !#zh
         构造函数，可查看 {{#crossLink "cc/vec2:method"}}cc.v2{{/crossLink}} 或者 {{#crossLink "cc/p:method"}}cc.p{{/crossLink}}
         @param out optional, the receiving vector

         @example
         ```js
         var v = cc.v2(10, 10);
         v.add(cc.v2(5, 5));      // return Vec2 {x: 15, y: 15};
         var v1;
         v.add(cc.v2(5, 5), v1);  // return Vec2 {x: 15, y: 15};
         ```
         */
        add(vector: Vec2, out: Vec2): Vec2 {
            out.x += vector.x;
            out.y += vector.y;
            return out;
        }

        /** !#en Subtracts one vector from this. If you want to save result to another vector, use sub() instead.
         !#zh 向量减法。如果你想保存结果到另一个向量，可使用 sub() 代替。
         @param vector !#en
         Constructor
         see {{#crossLink "cc/vec2:method"}}cc.v2{{/crossLink}} or {{#crossLink "cc/p:method"}}cc.p{{/crossLink}}
         !#zh
         构造函数，可查看 {{#crossLink "cc/vec2:method"}}cc.v2{{/crossLink}} 或者 {{#crossLink "cc/p:method"}}cc.p{{/crossLink}}

         @example
         ```js
         var v = cc.v2(10, 10);
         v.subSelf(cc.v2(5, 5));// return Vec2 {x: 5, y: 5};
         ```
         */
        subSelf(vector: Vec2): Vec2{
            var v:Vec2;
            v.x = vector.x - this.x;
            v.y = vector.y - this.y;
            return v;
        }

        /** !#en Subtracts one vector from this, and returns the new result.
         !#zh 向量减法，并返回新结果。
         @param vector !#en
         Constructor
         see {{#crossLink "cc/vec2:method"}}cc.v2{{/crossLink}} or {{#crossLink "cc/p:method"}}cc.p{{/crossLink}}
         !#zh
         构造函数，可查看 {{#crossLink "cc/vec2:method"}}cc.v2{{/crossLink}} 或者 {{#crossLink "cc/p:method"}}cc.p{{/crossLink}}
         @param out optional, the receiving vector

         @example
         ```js
         var v = cc.v2(10, 10);
         v.sub(cc.v2(5, 5));      // return Vec2 {x: 5, y: 5};
         var v1;
         v.sub(cc.v2(5, 5), v1);  // return Vec2 {x: 5, y: 5};
         ```
         */
        sub(vector:Vec2, out:Vec2): Vec2{
            out.x = vector.x - out.x ;
            out.y = vector.y - out.y;
            return out;
        }

        /** !#en Multiplies this by a number. If you want to save result to another vector, use mul() instead.
         !#zh 缩放当前向量。如果你想结果保存到另一个向量，可使用 mul() 代替。

         @example
         ```js
         var v = cc.v2(10, 10);
         v.mulSelf(5);// return Vec2 {x: 50, y: 50};
         ```
         */
        mulSelf(num: number): Vec2{

            this.x *= num;
            this.y *= num;
            return this;
        }

        /** !#en Multiplies by a number, and returns the new result.
         !#zh 缩放当前向量，并返回新结果。
         @param out optional, the receiving vector

         @example
         ```js
         var v = cc.v2(10, 10);
         v.mul(5);      // return Vec2 {x: 50, y: 50};
         var v1;
         v.mul(5, v1);  // return Vec2 {x: 50, y: 50};
         ```
         */
        mul(num: number, out:Vec2): Vec2{
            out.x = this.x * num;
            out.y = this.y * num;
            return out;
        }

        /** !#en Multiplies two vectors.
         !#zh 分量相乘。
         @param vector !#en
         Constructor
         see {{#crossLink "cc/vec2:method"}}cc.v2{{/crossLink}} or {{#crossLink "cc/p:method"}}cc.p{{/crossLink}}
         !#zh
         构造函数，可查看 {{#crossLink "cc/vec2:method"}}cc.v2{{/crossLink}} 或者 {{#crossLink "cc/p:method"}}cc.p{{/crossLink}}

         @example
         ```js
         var v = cc.v2(10, 10);
         v.scaleSelf(cc.v2(5, 5));// return Vec2 {x: 50, y: 50};
         ```
         */
        scaleSelf(vector: Vec2): Vec2{
            var v:Vec2 = this;
            v.x *= vector.x;
            v.y *= vector.y;
            return v;
        }

        /** !#en Multiplies two vectors, and returns the new result.
         !#zh 分量相乘，并返回新的结果。
         @param vector !#en
         Constructor
         see {{#crossLink "cc/vec2:method"}}cc.v2{{/crossLink}} or {{#crossLink "cc/p:method"}}cc.p{{/crossLink}}
         !#zh
         构造函数，可查看 {{#crossLink "cc/vec2:method"}}cc.v2{{/crossLink}} 或者 {{#crossLink "cc/p:method"}}cc.p{{/crossLink}}
         @param out optional, the receiving vector

         @example
         ```js
         var v = cc.v2(10, 10);
         v.scale(cc.v2(5, 5));      // return Vec2 {x: 50, y: 50};
         var v1;
         v.scale(cc.v2(5, 5), v1);  // return Vec2 {x: 50, y: 50};
         ```
         */
        scale(vector: Vec2, out: Vec2): Vec2{
            out.x = this.x * vector.x;
            out.y = this.y * vector.y;
            return out;
        }

        /** !#en Divides by a number. If you want to save result to another vector, use div() instead.
         !#zh 向量除法。如果你想结果保存到另一个向量，可使用 div() 代替。
         @param vector !#en
         Constructor
         see {{#crossLink "cc/vec2:method"}}cc.v2{{/crossLink}} or {{#crossLink "cc/p:method"}}cc.p{{/crossLink}}
         !#zh
         构造函数，可查看 {{#crossLink "cc/vec2:method"}}cc.v2{{/crossLink}} 或者 {{#crossLink "cc/p:method"}}cc.p{{/crossLink}}

         @example
         ```js
         var v = cc.v2(10, 10);
         v.divSelf(5); // return Vec2 {x: 2, y: 2};
         ```
         */
        divSelf(vector:Vec2): Vec2{
            this.x /= vector.x;
            this.y /= vector.y;
            return this;
        }

        /** !#en Divides by a number, and returns the new result.
         !#zh 向量除法，并返回新的结果。
         @param vector !#en
         Constructor
         see {{#crossLink "cc/vec2:method"}}cc.v2{{/crossLink}} or {{#crossLink "cc/p:method"}}cc.p{{/crossLink}}
         !#zh
         构造函数，可查看 {{#crossLink "cc/vec2:method"}}cc.v2{{/crossLink}} 或者 {{#crossLink "cc/p:method"}}cc.p{{/crossLink}}
         @param out optional, the receiving vector

         @example
         ```js
         var v = cc.v2(10, 10);
         v.div(5);      // return Vec2 {x: 2, y: 2};
         var v1;
         v.div(5, v1);  // return Vec2 {x: 2, y: 2};
         ```
         */
        div(num:number, out: Vec2): Vec2{
            out.x = this.x / num;
            out.y = this.y / num;
            return out;
        }

        /** !#en Negates the components. If you want to save result to another vector, use neg() instead.
         !#zh 向量取反。如果你想结果保存到另一个向量，可使用 neg() 代替。

         @example
         ```js
         var v = cc.v2(10, 10);
         v.negSelf(); // return Vec2 {x: -10, y: -10};
         ```
         */
        negSelf(): Vec2{
            this.x = -this.x;
            this.y = -this.y;
            return this;
        }

        /** !#en Negates the components, and returns the new result.
         !#zh 返回取反后的新向量。
         @param out optional, the receiving vector

         @example
         ```js
         var v = cc.v2(10, 10);
         var v1;
         v.neg(v1);  // return Vec2 {x: -10, y: -10};
         ```
         */
        neg(out:Vec2): Vec2{
            out.x = -out.x;
            out.y = -out.y;
            return out;
        }

        /** !#en Dot product
         !#zh 当前向量与指定向量进行点乘。
         @param vector !#en
         Constructor
         see {{#crossLink "cc/vec2:method"}}cc.v2{{/crossLink}} or {{#crossLink "cc/p:method"}}cc.p{{/crossLink}}
         !#zh
         构造函数，可查看 {{#crossLink "cc/vec2:method"}}cc.v2{{/crossLink}} 或者 {{#crossLink "cc/p:method"}}cc.p{{/crossLink}}

         @example
         ```js
         var v = cc.v2(10, 10);
         v.dot(cc.v2(5, 5)); // return 100;
         ```
         */
        dot(vector:Vec2): number{
            return this.x * vector.x + this.y * vector.y;
        }

        /** !#en Cross product
         !#zh 当前向量与指定向量进行叉乘。
         @param vector !#en
         Constructor
         see {{#crossLink "cc/vec2:method"}}cc.v2{{/crossLink}} or {{#crossLink "cc/p:method"}}cc.p{{/crossLink}}
         !#zh
         构造函数，可查看 {{#crossLink "cc/vec2:method"}}cc.v2{{/crossLink}} 或者 {{#crossLink "cc/p:method"}}cc.p{{/crossLink}}

         @example
         ```js
         var v = cc.v2(10, 10);
         v.cross(cc.v2(5, 5)); // return 0;
         ```
         */
        cross(vector:Vec2): number{
            return this.x * vector.y + this.y * vector.x;
        }

        /** !#en Returns the length of this vector.
         !#zh 返回该向量的长度。

         @example
         ```js
         var v = cc.v2(10, 10);
         v.mag(); // return 14.142135623730951;
         ```
         */
        mag(): number{
            var v = cc.pNormalize(cc.p(this.x,this.y));
            return this.x / v.x;
        }

        /** !#en Returns the squared length of this vector.
         !#zh 返回该向量的长度平方。

         @example
         ```js
         var v = cc.v2(10, 10);
         v.magSqr(); // return 200;
         ```
         */
        magSqr(): number{
            return this.x * this.x + this.y * this.y;
        }

        /** !#en Make the length of this vector to 1.
         !#zh 向量归一化，让这个向量的长度为 1。

         @example
         ```js
         var v = cc.v2(10, 10);
         v.normalizeSelf(); // return Vec2 {x: 0.7071067811865475, y: 0.7071067811865475};
         ```
         */
        normalizeSelf(): Vec2{
            var v = cc.pNormalize(cc.p(this.x,this.y)); // var len:number = Math.SQRT2(this.x * this.x +this.y * this.y);
            return new Vec2(v.x,v.y);
        }

        /** !#en
         Returns this vector with a magnitude of 1.<br/>
         <br/>
         Note that the current vector is unchanged and a new normalized vector is returned. If you want to normalize the current vector, use normalizeSelf function.
         !#zh
         返回归一化后的向量。<br/>
         <br/>
         注意，当前向量不变，并返回一个新的归一化向量。如果你想来归一化当前向量，可使用 normalizeSelf 函数。
         @param out optional, the receiving vector
         */
        normalize(out: Vec2): Vec2{
            var v = cc.pNormalize(cc.p(out.x,out.y));
            out.x = v.x;
            out.y = v.y;
            return out;
        }

        /** !#en Get angle in radian between this and vector.
         !#zh 夹角的弧度。
         @param vector !#en
         Constructor
         see {{#crossLink "cc/vec2:method"}}cc.v2{{/crossLink}} or {{#crossLink "cc/p:method"}}cc.p{{/crossLink}}
         !#zh
         构造函数，可查看 {{#crossLink "cc/vec2:method"}}cc.v2{{/crossLink}} 或者 {{#crossLink "cc/p:method"}}cc.p{{/crossLink}}
         */
        angle(vector: Vec2): number{
            //这里未实现
            return cc.pAngle(cc.p(this.x,this.y),cc.p(vector.x,vector.y));
        }

        /** !#en Get angle in radian between this and vector with direction.
         !#zh 带方向的夹角的弧度。
         @param vector !#en
         Constructor
         see {{#crossLink "cc/vec2:method"}}cc.v2{{/crossLink}} or {{#crossLink "cc/p:method"}}cc.p{{/crossLink}}
         !#zh
         构造函数，可查看 {{#crossLink "cc/vec2:method"}}cc.v2{{/crossLink}} 或者 {{#crossLink "cc/p:method"}}cc.p{{/crossLink}}
         */
        signAngle(vector: Vec2): number{
            //这里未实现
            return cc.pAngleSigned(cc.p(this.x,this.y),cc.p(vector.x,vector.y));
        }

        /** !#en rotate
         !#zh 返回旋转给定弧度后的新向量。
         @param out optional, the receiving vector
         */
        rotate(radians: number, out:Vec2): Vec2{
            var v = cc.pRotateByAngle(cc.p(out.x,out.y),cc.p(0,0),radians);
            out.x = v.x;
            out.y = v.y;
            return out;
        }

        /** !#en rotate self
         !#zh 按指定弧度旋转向量。
         */
        rotateSelf(radians: number): Vec2{
            var v = cc.pRotateByAngle(cc.p(this.x,this.y),cc.p(0,0),radians);
            this.x = v.x;
            this.y = v.y;
            return this;
        }

        /** !#en return a Vec2 object with x = 1 and y = 1.
         !#zh 新 Vec2 对象。 */
        ONE: Vec2;
        /** !#en return a Vec2 object with x = 0 and y = 0.
         !#zh 返回 x = 0 和 y = 0 的 Vec2 对象。 */
        ZERO: Vec2;
        /** !#en return a Vec2 object with x = 0 and y = 1.
         !#zh 返回 x = 0 和 y = 1 的 Vec2 对象。 */
        UP: Vec2;
        /** !#en return a Vec2 object with x = 1 and y = 0.
         !#zh 返回 x = 1 和 y = 0 的 Vec2 对象。 */
        RIGHT: Vec2;
    }
}
