/**
 * Created by w on 2017/4/18.
 */
module behavior{
    import baseRole = game.baseRole;
    export enum e_Type{
        e_node,
        e_actionNode,       //动作节点
        e_conditionNode,    //条件节点
        e_behaviorNode,     //决策节点
    }
    export enum e_State{
        e_Waiting,          //等待执行
        e_Executing,        //运行中
        e_Completed,        //已完成
    }
    export enum e_condition_Type{
        e_dayu,
        e_dengyu,
        e_xiaoyu,
    }
    //所有节点的父类
    export class Node{
        m_Parent:Node = null            //父节点
        m_Children:Array<Node> = null   //子对象链表
        m_name:string =   "Node"            //类名
        m_Type:e_Type = null            //节点类型
        m_State:e_State = null          //当前状态
        constructor( ){
            this.m_Children = new Array<Node>()
        }
        addChild( child:Node ){
            child.setParent( this )
            this.m_Children.push( child )
        }
        setParent( parent:Node ){
            this.m_Parent = parent
        }
        Enter( data:any ){
            this.m_State = e_State.e_Executing
        }
        Exit( data:any ){
            this.m_State = e_State.e_Waiting
        }
        child_Complete_Callback( child:Node ){
            child.Exit.call( child , null )
        }
        Execute(){

        }
        getState():e_State{
            return this.m_State
        }
        shutdown() {
        }

        update( t ){
            if(this.m_State == e_State.e_Completed ){
                if( this.m_Parent ){
                    this.m_Parent.child_Complete_Callback(this)
                }
            }
        }
    }
    //条件节点
    export class condition_struct{
        m_func:Function = null      //这里传入的参数必须是比较型的 function():boolean{ player.x > 100 }
        m_condition:Node = null
        m_Target:Object = null
        constructor( f:Function , c:Node , t:Object){
            this.m_func = f
            this.m_condition = c
            this.m_Target = t
        }
    }

    export class condition_Node extends Node {
        m_conditionArray: Array<condition_struct> = null

        m_ControlNode: Node = null
        m_ControlRole: baseRole = null
        static create(condition ,role:baseRole) {
            return new condition_Node(condition ,role)
        }
        constructor(condition,role:baseRole) {
            super()
            this.m_conditionArray = new Array<condition_struct>()
            this.m_Type = e_Type.e_conditionNode
            this.m_ControlRole = role
            this.m_ControlNode = null
        }
        // 创建时的当前条件为传入条件，则执行对应节点
        register( data:condition_struct) {
            data.m_condition.setParent( this )
            this.m_conditionArray.push(data)
        }
        SelectConditionNode():boolean {
            for( let k = 0; k < this.m_conditionArray.length; k++){
                let data:condition_struct = this.m_conditionArray[k]
                //如果当前条件返回结果为true 选择该条件的节点做为执行节点 需要考虑优先级
                if(data.m_func.call(data.m_Target)){
                    this.m_ControlNode = data.m_condition
                    this.m_ControlNode.Enter( this.m_ControlRole )
                    return true
                }
            }
            return false
        }
        child_Complete_Callback( child:Node ){
            child.Exit( null )
            this.m_ControlNode = null
            if(this.SelectConditionNode()  == false ){
                //没有可执行条件
            }
        }
        update(t) {
            if (this.m_ControlNode != null) {
                this.m_ControlNode.update(t)
                //如果当前节点条件已执行完毕 重新选择符合条件的节点
                if(this.m_ControlNode.m_State == e_State.e_Completed ){
                    this.m_ControlNode = null
                }
            }
            else{
                if(this.SelectConditionNode() == false){
                    //当前条件中没有可执行的条件
                }
            }
        }
    }

    export class Behavior_Tree{
        static create(){
            return new Behavior_Tree()
        }
        m_Children:Array<Node> = null
        m_ControlNode:Node = null     //当前控制的节点
        m_ControlRole:baseRole = null   //当前控制的对象
        constructor(){
            this.m_Children = new Array<Node>()
        }
        getChildByname( name:string ):Node{
            for(let k = 0; k < this.m_Children.length; k++){
                if(this.m_Children[k].m_name === name){
                    return this.m_Children[k]
                }
            }
            return null
        }
        addChild( node:Node ){
            //node.setParent( this )
            this.m_Children.push( node )
        }
        update ( t ){
            if(this.m_ControlNode == null){

            }else{
                this.m_ControlNode.update( t )
            }
        }
    }
}