module game {

    export class Map <K, V> {
        m_keys: Array<K> = null;
        _data: Object = null;
        size: number = 0;
        _id: number = 0 //多个map时用到
        constructor() {
            this.m_keys = new Array();
            this._data = new Object();
        }

        setId(id:number){
            this._id = id
        }
        getId():number{
            return this._id
        }
        forEach(callbackfn: (value: V, index: K, map: Map<K, V>) => void, thisArg: any): void {
            if (typeof callbackfn != 'function') {
                return;
            }
            if (thisArg == null) {
                return;
            }

            var len = this.m_keys.length;
            for (var i = 0; i < len; i++) {
                var k: any = this.m_keys[i];
                callbackfn.call(thisArg, this._data[k], k, this);
            }
        }


        delete(key: K): boolean {
            for (var i = 0; i < this.m_keys.length; i++) {
                if (key === this.m_keys[i]) {
                    var del_keys = this.m_keys.splice(i, 1);
                    for (var k in del_keys) {
                        var data: any = del_keys[k];
                        this._data[data] = null;
                    }
                    this.size = this.m_keys.length;
                    return true;
                }
            }

            return false;
        }

        getKeyByIndex( index:number ):K
        {
            for (var i = 0; i < this.m_keys.length; i++) {
                if (index == i) {
                    var keybuff: any = this.m_keys[i];
                    return keybuff;
                }
            }
            return null;
        }

        getByIndex( index:number ):V
        {
            for (var i = 0; i < this.m_keys.length; i++) {
                if (index == i) {
                    var keybuff: any = this.m_keys[i];
                    return this._data[keybuff];
                }
            }
            return null;
        }

        // /**
        //  * 获取某键对应的值
        //  * @param {String} key
        //  * @return {Object} value
        //  */
        get(key: K): V {
            for (var i = 0; i < this.m_keys.length; i++) {
                if (key === this.m_keys[i]) {
                    var keybuff: any = key;
                    return this._data[keybuff];
                }
            }
            return null;
        }

        getKeyByData( data:any ){
            for (var i = 0; i < this.m_keys.length; i++) {
                var keybuff: any = this.m_keys[i];
                if( this._data[keybuff] === data ){
                    return keybuff
                }
            }
            return null;
        }

        has(key: K): boolean {
            var ret = this.get(key);
            if (ret) {
                return true;
            }
            return false;
        }

        set(key: K, value: V) {
            this.delete(key); // 替换掉旧的数据
            this.m_keys.push(key);
            var keybuff: any = key;
            this._data[keybuff] = value;
            this.sort();
            this.size = this.m_keys.length;
            return true;
        }


        //排序
        sort() {
            // this.m_keys.sort( function (a:K, b:K) {
            //     return b < a;
            // });
        }


        //删除所有数据
        clear() {
            this.m_keys = null;
            this._data = null;
            this.m_keys = new Array();
            this._data = new Object();
            this.size = 0;
        }


        /**
         * 判断Map是否为空
         */
        isEmpty() {
            return this.m_keys.length == 0;
        }


    }


    export class ObjMap <  V> {
        m_keys: Array<any> = null;
        _data: Object = null;
        size: number = 0;

        constructor(obj: Object = null) {
            this.m_keys = new Array();
            this._data = new Object();

            if (obj) {
                for (var p in obj) {
                    var data = obj[p];
                    this.set(p, data);
                }
            }
        }

        getObject(): Object {
            var obj: Object = new Object();
            this.forEach(( v,k ) => {
                obj[k] = v;
            }, this)
            return obj;
        }


        forEach(callbackfn: (value: V, index: any, map: ObjMap< V>) => void, thisArg: any): void {
            if (typeof callbackfn != 'function') {
                return;
            }
            if (thisArg == null) {
                return;
            }

            var len = this.m_keys.length;
            for (var i = 0; i < len; i++) {
                var k: any = this.m_keys[i];
                callbackfn.call(thisArg, this._data[k], k, this);
            }
        }


        delete(key: any): boolean {
            for (var i = 0; i < this.m_keys.length; i++) {
                if (key === this.m_keys[i]) {
                    var del_keys = this.m_keys.splice(i, 1);
                    for (var k in del_keys) {
                        var data: any = del_keys[k];
                        this._data[data] = null;
                        this.size = this.m_keys.length;
                    }
                    return true;
                }
            }

            return false;
        }

        // /**
        //  * 获取某键对应的值
        //  * @param {String} key
        //  * @return {Object} value
        //  */
        get(key: any): V {
            for (var i = 0; i < this.m_keys.length; i++) {
                if (key === this.m_keys[i]) {
                    var keybuff: any = key;
                    return this._data[keybuff];
                }
            }
            return null;
        }

        has(key: any): boolean {
            var ret = this.get(key);
            if (ret) {
                return true;
            }
            return false;
        }

        set(key: any, value: V) {
            this.delete(key); // 替换掉旧的数据
            this.m_keys.push(key);
            var keybuff: any = key;
            this._data[keybuff] = value;
            this.sort();
            this.size = this.m_keys.length;
            return true;
        }


        //排序
        sort() {
            // this.m_keys.sort( function (a:K, b:K) {
            //     return b < a;
            // });
        }


        //删除所有数据
        clear() {
            this.m_keys = null;
            this._data = null;
            this.m_keys = new Array();
            this._data = new Object();
            this.size = 0;
        }


        /**
         * 判断Map是否为空
         */
        isEmpty() {
            return this.m_keys.length == 0;
        }


    }


}
//# sourceMappingURL=MapGame.js.map