/**
 * Created by w on 2017/3/31.
 */
var cc;
(function (cc) {
    var Vec2 = (function () {
        function Vec2(x, y) {
            this.x = x;
            this.y = y;
        }
        Vec2.getAngleByV2 = function (nowV, parentV) {
            return cc.pAngle(nowV, parentV);
        };
        /** !#en TODO
         !#zh 当前的向量是否与指定的向量相等。
         @param other !#en
         Constructor
         see {{#crossLink "cc/vec2:method"}}cc.v2{{/crossLink}} or {{#crossLink "cc/p:method"}}cc.p{{/crossLink}}
         !#zh
         构造函数，可查看 {{#crossLink "cc/vec2:method"}}cc.v2{{/crossLink}} 或者 {{#crossLink "cc/p:method"}}cc.p{{/crossLink}}
         */
        Vec2.prototype.equals = function (other) {
            return this.x == other.x && this.y == other.y;
        };
        /** !#en Adds this vector. If you want to save result to another vector, use add() instead.
         !#zh 向量加法。如果你想保存结果到另一个向量，使用 add() 代替。
         @param vector !#en
         Constructor
         see {{#crossLink "cc/vec2:method"}}cc.v2{{/crossLink}} or {{#crossLink "cc/p:method"}}cc.p{{/crossLink}}
         !#zh
         构造函数，可查看 {{#crossLink "cc/vec2:method"}}cc.v2{{/crossLink}} 或者 {{#crossLink "cc/p:method"}}cc.p{{/crossLink}}

         @example
         ```js
         var v = cc.v2(10, 10);
         v.addSelf(cc.v2(5, 5));// return Vec2 {x: 15, y: 15};
         ```
         */
        Vec2.prototype.addSelf = function (other) {
            this.x += other.x;
            this.y += other.y;
            return this;
        };
        /** !#en Adds two vectors, and returns the new result.
         !#zh 向量加法，并返回新结果。
         @param vector !#en
         Constructor
         see {{#crossLink "cc/vec2:method"}}cc.v2{{/crossLink}} or {{#crossLink "cc/p:method"}}cc.p{{/crossLink}}
         !#zh
         构造函数，可查看 {{#crossLink "cc/vec2:method"}}cc.v2{{/crossLink}} 或者 {{#crossLink "cc/p:method"}}cc.p{{/crossLink}}
         @param out optional, the receiving vector

         @example
         ```js
         var v = cc.v2(10, 10);
         v.add(cc.v2(5, 5));      // return Vec2 {x: 15, y: 15};
         var v1;
         v.add(cc.v2(5, 5), v1);  // return Vec2 {x: 15, y: 15};
         ```
         */
        Vec2.add = function (v1, v2) {
            var result = new Vec2(0, 0);
            result.addSelf(v1);
            result.addSelf(v2);
            return result;
        };
        ;
        /** !#en Subtracts one vector from this. If you want to save result to another vector, use sub() instead.
         !#zh 向量减法。如果你想保存结果到另一个向量，可使用 sub() 代替。
         @param vector !#en
         Constructor
         see {{#crossLink "cc/vec2:method"}}cc.v2{{/crossLink}} or {{#crossLink "cc/p:method"}}cc.p{{/crossLink}}
         !#zh
         构造函数，可查看 {{#crossLink "cc/vec2:method"}}cc.v2{{/crossLink}} 或者 {{#crossLink "cc/p:method"}}cc.p{{/crossLink}}

         @example
         ```js
         var v = cc.v2(10, 10);
         v.subSelf(cc.v2(5, 5));// return Vec2 {x: 5, y: 5};
         ```
         */
        Vec2.prototype.subSelf = function (vector) {
            this.x -= vector.x;
            this.y -= vector.y;
            return this;
        };
        /** !#en Subtracts one vector from this, and returns the new result.
         !#zh 向量减法，并返回新结果。
         @param vector !#en
         Constructor
         see {{#crossLink "cc/vec2:method"}}cc.v2{{/crossLink}} or {{#crossLink "cc/p:method"}}cc.p{{/crossLink}}
         !#zh
         构造函数，可查看 {{#crossLink "cc/vec2:method"}}cc.v2{{/crossLink}} 或者 {{#crossLink "cc/p:method"}}cc.p{{/crossLink}}
         @param out optional, the receiving vector

         @example
         ```js
         var v = cc.v2(10, 10);
         v.sub(cc.v2(5, 5));      // return Vec2 {x: 5, y: 5};
         var v1;
         v.sub(cc.v2(5, 5), v1);  // return Vec2 {x: 5, y: 5};
         ```
         */
        Vec2.sub = function (v1, v2) {
            var result = new Vec2(0, 0);
            result.x = v1.x - v2.x;
            result.y = v1.y - v2.y;
            return result;
        };
        ;
        /** !#en Multiplies this by a number. If you want to save result to another vector, use mul() instead.
         !#zh 缩放当前向量。如果你想结果保存到另一个向量，可使用 mul() 代替。

         @example
         ```js
         var v = cc.v2(10, 10);
         v.mulSelf(5);// return Vec2 {x: 50, y: 50};
         ```
         */
        Vec2.mul = function (v1, num) {
            var result = new Vec2(0, 0);
            result.x = v1.x * num;
            result.y = v1.y * num;
            return result;
        };
        ;
        Vec2.div = function (v1, num) {
            var result = new Vec2(0, 0);
            result.x = v1.x / num;
            result.y = v1.y / num;
            return result;
        };
        /** !#en Negates the components. If you want to save result to another vector, use neg() instead.
         !#zh 向量取反。如果你想结果保存到另一个向量，可使用 neg() 代替。

         @example
         ```js
         var v = cc.v2(10, 10);
         v.negSelf(); // return Vec2 {x: -10, y: -10};
         ```
         */
        Vec2.prototype.negSelf = function () {
            var result = new Vec2(0, 0);
            result.x = -this.x;
            result.y = -this.y;
            return result;
        };
        ;
        /** !#en Returns the length of this vector.
         !#zh 返回该向量的长度。

         @example
         ```js
         var v = cc.v2(10, 10);
         v.mag(); // return 14.142135623730951;
         ```
         */
        Vec2.pLength = function (v1, v2) {
            return cc.pLength(Vec2.sub(v1, v2));
        };
        ;
        /** !#en
         Returns this vector with a magnitude of 1.<br/>
         <br/>
         Note that the current vector is unchanged and a new normalized vector is returned. If you want to normalize the current vector, use normalizeSelf function.
         !#zh
         返回归一化后的向量。<br/>
         <br/>
         注意，当前向量不变，并返回一个新的归一化向量。如果你想来归一化当前向量，可使用 normalizeSelf 函数。
         @param out optional, the receiving vector
         */
        Vec2.prototype.normalize = function () {
            return cc.pNormalize(this);
        };
        ;
        /** !#en Get angle in radian between this and vector.
         !#zh 夹角的弧度。
         @param vector !#en
         Constructor
         see {{#crossLink "cc/vec2:method"}}cc.v2{{/crossLink}} or {{#crossLink "cc/p:method"}}cc.p{{/crossLink}}
         !#zh
         构造函数，可查看 {{#crossLink "cc/vec2:method"}}cc.v2{{/crossLink}} 或者 {{#crossLink "cc/p:method"}}cc.p{{/crossLink}}
         */
        Vec2.prototype.angle = function (vector) {
            return 0;
        };
        ;
        return Vec2;
    }());
    cc.Vec2 = Vec2;
})(cc || (cc = {}));
//# sourceMappingURL=Vector2.js.map